@@ -93,7 +93,7 @@ end
 
 
 local function rate_limit(conf, ctx)
-    return limit_count.rate_limit(conf, ctx)
+    return limit_count.rate_limit(conf, ctx, "limit-count", 1)
 end
 
 
